﻿Type=StaticCode
Version=1.8
ModulesStructureVersion=1
B4R=true
@EndOfDesignText@
Private Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'Public variables can be accessed from all modules.
	Type AsyncStreamsWrapper (EnclosedObject As AsyncStreams)
	Type WiFiSocketWrapper (EnclosedObject As WiFiSocket)
	
	Private const MapSize As Byte = 16
	
	Private FreePosition As Byte = 0
	Private GlobalKeyObject(16) As AsyncStreams
	Private GlobalValueObject(16) As WiFiSocket
End Sub

public Sub Init
	' Initialize the globals with well-defined values
	Dim i As Byte

	
	' Null the object arrays
	For i = 0 To (MapSize - 1)
		GlobalKeyObject(i) = Null
		GlobalValueObject(i) = Null
	Next
	
	FreePosition = 0
End Sub

Public Sub Put(KeyObject As AsyncStreams, ValueObject As WiFiSocket) As Boolean
	' Store is full
	If FreePosition = MapSize Then
		Return(False)
	End If
	
	' Only store an object if it doesn't exist in the store
	If GetPos(KeyObject) <> -1 Then
		Return(True)
	End If
	
	' Store key and value at the next free position (always on the top)
	GlobalKeyObject(FreePosition) = KeyObject
	GlobalValueObject(FreePosition) = ValueObject
	FreePosition = FreePosition + 1
	
	Return(True)
End Sub

Public Sub Get(KeyObject As AsyncStreams, ValueObject As WiFiSocketWrapper) As Boolean
	Dim i As Byte


	ValueObject.EnclosedObject = Null
	
	' Store is empty
	If FreePosition = 0 Then
		Return(False)
	End If
	
	' Try to find the given object by searching in all keys
	For i = 0 To (FreePosition - 1)
		If KeyObject = GlobalKeyObject(i) Then
			ValueObject.EnclosedObject = GlobalValueObject(i)
			Return(True)
		End If
	Next
	
	Return(False)
End Sub

Public Sub GetAtPos(Pos As Byte, KeyObject As AsyncStreamsWrapper, ValueObject As WiFiSocketWrapper) As Boolean
	' Initialize the return values
	KeyObject.EnclosedObject = Null
	ValueObject.EnclosedObject = Null
	
	' Store is empty
	If FreePosition = 0 Then
		Return(False)
	End If

	' No object at position
	If Pos >= FreePosition Then
		Return(False)	
	End If

	KeyObject.EnclosedObject = GlobalKeyObject(Pos)	
	ValueObject.EnclosedObject = GlobalValueObject(Pos)
	Return(True)
End Sub

Public Sub Remove(KeyObject As AsyncStreams) As Boolean
	Dim i As Byte

	
	' Store is empty
	If FreePosition = 0 Then
		Return(False)
	End If
	
	' Try to find the given object by searching in all keys
	' If found, take the topmost object and shift it in the deleted position. No hole in the store, because we fill it with the topmost object an increment the free position.
	For i = 0 To (FreePosition - 1)
		If KeyObject = GlobalKeyObject(i) Then
			GlobalKeyObject(i) = GlobalKeyObject(FreePosition - 1)
			GlobalValueObject(i) = GlobalValueObject(FreePosition - 1)
			FreePosition = FreePosition - 1
			Return(True)
		End If
	Next
	
	Return(False)
End Sub

Public Sub FindFirstByValue(ValueObject As WiFiSocket, KeyObject As AsyncStreamsWrapper) As Boolean
	Dim i As Byte

	
	KeyObject.EnclosedObject = Null
	
	' Store is empty
	If FreePosition = 0 Then
		Return(False)
	End If
	
	' Try to find the given object by searching in the values. Values are not necessarily unique. That means we can only find the first occurrence.
	For i = 0 To (FreePosition - 1)
		If ValueObject = GlobalValueObject(i) Then
			KeyObject.EnclosedObject = GlobalKeyObject(i)
			Return(True)
		End If
	Next
	
	Return(False)
End Sub

Public Sub Count As Byte
	' The fill size of the store
	Return(FreePosition)
End Sub

Public Sub GetPos(KeyObject As AsyncStreams) As Int
	Dim i As Byte
	
	
	' Store is empty
	If FreePosition = 0 Then
		Return(-1)
	End If
	
	' Try to find the given object by searching in all keys.
	' And return the position.
	For i = 0 To (FreePosition - 1)
		If KeyObject = GlobalKeyObject(i) Then
			Return(i)
		End If
	Next
	
	Return(-1)
End Sub

Public Sub GetNextPos As Int
	' Store is full
	If FreePosition = MapSize Then
		Return(-1)
	End If
	
	Return(FreePosition)
End Sub
